# R2WBwrapper.R: a R2WinBUGS wrapper
# KUBO Takuya (kubo@ees.hokudai.ac.jp)
# DATE: 2008-03-22 16:01:33 +0900

library(R2WinBUGS)
library(coda)

setG <- function(name, x) assign(name, x, envir = .GlobalEnv)

# Clear ==================================================================
clear.v.datanames <- function()
{
	setG("v.datanames", NULL)
}
clear.param <- function()
{
	setG("parameters.to.save", NULL)
	setG("list.inits", NULL)
}
clear.data.param <- function()
{
	clear.v.datanames()
	clear.param()
}

# !!! Initialization !!!
clear.data.param()

# Data ===================================================================
set.data.sub <- function(name, val)
{
	if (is.null(val)) stop("# ERROR: no data for", name, "\n")
	ditem <- val
	setG(name, ditem)
}

set.data <- function(name = NA, val = NA, list.data = NA)
{
	if (!is.na(list.data)) {
		v.datanames <- c(v.datanames, names(list.data))
		setG("v.datanames", v.datanames)
		for (name in v.datanames) set.data.sub(name, list.data[[name]])
	} else if (!is.na(name)) {
		v.datanames <- c(v.datanames, name)
		setG("v.datanames", v.datanames)
		if (length(dim(val)) < 2) val <- as.vector(val)
		set.data.sub(name, val)
	} else {
		stop("# Error: set either (name, val)  or list.data")
	}
}

# Parameters =============================================================

set.param <- function(name, val = NA, save = TRUE)
{
	parameters.to.save <- get("parameters.to.save", envir = .GlobalEnv)
	list.inits <- get("list.inits", envir = .GlobalEnv)
	if (save) {
		parameters.to.save <- c(parameters.to.save, name)
	}
	if (!is.na(val[1])) {
		if (length(dim(val)) < 2) val <- as.vector(val)
		list.new <- list(val)
		names(list.new) <- name
		list.inits <- c(list.inits, list.new)
	}
	setG("parameters.to.save", parameters.to.save)
	setG("list.inits", list.inits)
}

# Calling WinBUGS ========================================================
call.bugs <- function(
	file.bug = "model.bug.txt",
	debug = TRUE,
	n.chains = 3,
	n.iter, n.burnin, n.thin
) {
	model.file <- paste(getwd(), file.bug, sep = "/")
	inits <- function() list.inits
	if (Sys.info()["sysname"] == "Linux") {
		WINEPATH <- "/usr/local/bin/winepath"
		sys.time <- system.time(
			post.bugs <- bugs(
				data = v.datanames,
				inits = inits,
				parameters.to.save = parameters.to.save,
				model.file = model.file,
				n.chains = n.chains, n.iter = n.iter,
				n.burnin = n.burnin, n.thin = n.thin,
				bugs.directory = paste(
					Sys.getenv("HOME"),
					".wine/drive_c/Program\ Files/WinBUGS14/",
					sep = "/"
				),
				working.directory = NULL,
				clearWD = TRUE,
				useWINE = TRUE,
				newWINE = TRUE,
				WINE = "/usr/local/bin/wine",
				WINEPATH = WINEPATH,
				debug = debug
			)
		)
		print(sys.time)
	} else {
		post.bugs <- bugs(
			v.datanames, inits, parameters.to.save, model.file,
			n.chains = n.chains, n.iter = n.iter,
			n.burnin = n.burnin, n.thin = n.thin,
			bugs.directory = "c:/Program Files/WinBUGS14/",
			working.directory = NULL,
			clearWD = TRUE,
			debug = debug
		)
	}
	clear.data.param() # !!! !!! !!! !!! !!! !!! !!! !!! !!! !!! !!! !!!
	post.bugs
}

# Post processings =======================================================
to.list <- function(x.bugs) mcmc.list(
	lapply(
		1:x.bugs$n.chain,
		function(chain) as.mcmc(x.bugs$sims.array[, chain,])
	)
)

to.mcmc <- function(x.bugs) as.mcmc(x.bugs$sims.matrix)

pl <- function(key, x.bugs = post.bugs, ...)
{
	cn <- colnames(to.mcmc(x.bugs))
	if (!exists("post.list")) post.list <- to.list(x.bugs)
	if (length(key) == 1) {
		plot(post.list[,grep(key, cn),], ask = T, smooth = F, ...)
	} else {
		plot(post.list[,cn %in% key,], ask = T, smooth = F, ...)
	}
}

choose.post <- function(key, alpha = 0.05, x.bugs = post.bugs)
{
	x.mcmc <- to.mcmc(x.bugs)
	c.mcmc <- x.mcmc[, grep(key, colnames(x.mcmc))]
	for (i in colnames(c.mcmc)) {
		v.prob <- quantile(
			c.mcmc[, i],
			probs = c(alpha * 0.05, 0.50, 1 - alpha * 0.5)
		)
		if (v.prob[1] * v.prob[3] > 0) {
			cat(sprintf(
				"%s\t%8.2f%8.2f%8.2f\n",
				i, v.prob[1], v.prob[2], v.prob[3]
			))
		}
	}
}
